<?php

/***************************************************************
*  Copyright notice
*
*  (c) 2007 Stefan Galinski <Stefan.Galinski@frm2.tum.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once('res/csstidy/class.csstidy.php');

/**
 * Contains the code to manipulate the includeCSS section of the tsconfig.
 */
class scriptmerger
{
	var $cachePath = 'typo3temp/scriptmerger/';

	/**
	 * Constructor
	 *
	 * Contains the code to manipulate the includeCSS section of the tsconfig.
	 *
	 * @return bool true or false
	 */
	function scriptmerger()
	{
		// get includeCSS files
		$cssFiles = array();
		if(is_array($GLOBALS['TSFE']->pSetup['includeCSS.'])) {
			foreach($GLOBALS['TSFE']->pSetup['includeCSS.'] as $key=>$file) {
				if($key[strlen($key)-1] == '.')
					continue;

				$file =  $GLOBALS['TSFE']->tmpl->getFileName($file);
				if(!is_file($file))
					continue;

				$media = $GLOBALS['TSFE']->pSetup['includeCSS.'][$key . '.']['media'];
				$media = (!empty($media) ? $media : 'screen');
				$cssFiles[$media][] = $file;
			}
		}

		// get css inlineStyle2TempFile
		if($GLOBALS['TSFE']->config['config']['inlineStyle2TempFile']) {
			if(is_array($GLOBALS['TSFE']->tmpl->setup['plugin.'])) {
				$temp_styleLines = array();
				foreach($GLOBALS['TSFE']->tmpl->setup['plugin.'] as $key=>$iCSScode) {
					if(!is_array($iCSScode) || !$iCSScode['_CSS_DEFAULT_STYLE'])
						continue;

					$temp_styleLines[] = '/* default styles for extension "' .
						substr($key, 0, -1) . '" */' . chr(10) . $iCSScode['_CSS_DEFAULT_STYLE'];
					unset($GLOBALS['TSFE']->tmpl->setup['plugin.'][$key]['_CSS_DEFAULT_STYLE']);
				}
			}

			if (count($temp_styleLines)) {
				$temp_styleLines = implode(chr(10), $temp_styleLines);
				$script = 'typo3temp/stylesheet_' . substr(md5($temp_styleLines), 0, 10) . '.css';

				// write file
				if(!is_file(PATH_site . $script))
					t3lib_div::writeFile(PATH_site . $script, $temp_styleLines);
				$cssFiles['screen'][] = $script;
			}
		}

		if(!count($cssFiles))
			return false;

		// create directory
		if(!is_dir(PATH_site . $this->cachePath))
			t3lib_div::mkdir(PATH_site . $this->cachePath);

		// get cached files
		$files = t3lib_div::getFilesInDir(PATH_site . $this->cachePath);

		// drop config array
		unset($GLOBALS['TSFE']->pSetup['includeCSS.']);

		// check mtimes of files
		foreach($cssFiles as $media=>$files) {
			$mTimes = 0;
			foreach($files as $cssFile)
				$mTimes += filemtime($cssFile);

			if(!array_key_exists($mTimes . '.css', $files)) {
				$fileContent = '';
				foreach($files as $cssFile)
					$fileContent .= $this->replaceURL(file_get_contents(PATH_site . $cssFile), $cssFile);

				// css optimizations by css tidy (failsafe mode)
				$csstidy = new csstidy();
				$csstidy->set_cfg('preserve_css', true);
				$csstidy->set_cfg('remove_last_;', true);
				$csstidy->set_cfg('timestamp', true);
				$csstidy->parse($fileContent);
				$fileContent = $csstidy->print->plain();

				t3lib_div::writeFile(PATH_site . $this->cachePath . $mTimes . '.css', $fileContent);
			}

			$GLOBALS['TSFE']->pSetup['includeCSS.']['scriptmerger_' . $media] =
				$this->cachePath . $mTimes . '.css';
			$GLOBALS['TSFE']->pSetup['includeCSS.']['scriptmerger_' . $media . '.']['media'] =
				$media;
		}

		return true;
	}

	/**
	 * Replaces all occurences of url in the css code by new paths
	 *
	 * @param string $content css code
	 * @param string $cssFile css file
	 * @return string fixed css code
	 */
	function replaceURL($content, $cssFile)
	{
		$base = '../../' . dirname($cssFile);
		$pattern = '/(.+)url\(["|\']?([a-z0-9_\.\/\\\]+)["|\']?\)(.+)/i';
		$replacement = '\1url("' . $base . '/\2")\3';
		return preg_replace($pattern, $replacement, $content, -1);
	}
}

?>
